<?php

declare(strict_types=1);

/**
 * @author Amasty Team
 * @copyright Copyright (c) Amasty (https://www.amasty.com)
 * @package Blog Pro Sitemap Compatibility
 */

namespace Amasty\BlogSitemap\Model\Sitemap\ItemProvider;

use Amasty\Blog\Model\Posts;
use Amasty\Blog\Model\ResourceModel\Posts\CollectionFactory;
use Magento\Framework\DataObject;
use Magento\Framework\DataObjectFactory;
use Magento\Sitemap\Model\ItemProvider\ItemProviderInterface;
use Magento\Sitemap\Model\SitemapItemInterfaceFactory;

class Post implements ItemProviderInterface
{
    /**
     * @var CollectionFactory
     */
    private $collectionFactory;

    /**
     * @var SitemapItemInterfaceFactory
     */
    private $itemFactory;

    /**
     * @var DataObjectFactory
     */
    private $dataObjectFactory;

    public function __construct(
        CollectionFactory $collectionFactory,
        SitemapItemInterfaceFactory $itemFactory,
        DataObjectFactory $dataObjectFactory
    ) {
        $this->collectionFactory = $collectionFactory;
        $this->itemFactory = $itemFactory;
        $this->dataObjectFactory = $dataObjectFactory;
    }

    public function getItems($storeId)
    {
        $storeId = (int)$storeId;

        $collection = $this->collectionFactory->create();
        $collection->addStoreWithDefault($storeId);
        $collection->addFilterByStatus();
        $collection->setDateOrder();

        return array_map(function (Posts $item) {
            return $this->itemFactory->create([
                'url' => $item->getRelativeUrl(),
                'updatedAt' => $item->getUpdatedAt(),
                'images' => $this->getImages($item),
                'priority' => '0.5',
                'changeFrequency' => 'daily',
            ]);
        }, $collection->getItems());
    }

    private function getImages(Posts $item): ?DataObject
    {
        if (!$item->getPostImageSrc()) {
            return null;
        }

        return $this->getDataObject([
            'collection' => [$this->getDataObject(['url' => $item->getPostImageSrc()])],
            'title' => $item->getTitle(),
            'thumbnail' => $item->getPostThumbnailSrc()
        ]);
    }

    private function getDataObject(array $data): DataObject
    {
        return $this->dataObjectFactory->create(['data' => $data]);
    }
}
